/*
    -----------------------------------------------------------------------------

                   --- CAEN SpA - Computing Systems Division ---

    -----------------------------------------------------------------------------

    CAENRFIDLib.h

    -----------------------------------------------------------------------------

	Author: Alberto Lucchesi (a.lucchesi@caen.it)  CAEN Engineering srl
	Rev : Antoine Roguez (aroguez@laas.fr) LAAS CNRS

	Created: Oct 2006
	Rev : Aug 2007

	Version 3.2L


    -----------------------------------------------------------------------------
*/

#ifndef __CAENRFIDLIB_H
#define __CAENRFIDLIB_H
#define byte unsigned char

#include "CAENRFIDTypes.h"
#include <termios.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus*/


/*fonctions de tests*/
void test_set_RTS(CAENRFIDHandle *Handle);
int switch_RTS (CAENRFIDHandle Handle);
/*fin de fonctions de tests*/

int setRTSHigh(CAENRFIDHandle Handle);

int setRTSLow(CAENRFIDHandle Handle);

/*
        connect_RS232
        -----------------------------------------------------------------------------
        Parameters:
	    [in]  port 			: The COM port on which the reader is connected
									(eg. 1 for COM1)
	    [in]  speed			: The speed of the connection
									(defined in termios.h, B115200 for example)
	    [in]  parity		: The parity of the connection : 
									E for even, O for odd, N for no
	    [in]  bytesize		: The number of bytes sent each time
									(defined in termios.h, CS8 for example)
	    [in]  usb2serial            : 1 if using an usb2serial converter
            [out] Handle    	        : The handle that identifies the device.
        -----------------------------------------------------------------------------
        Returns:
            An error code about the execution of the function.
        -----------------------------------------------------------------------------
        Description:
            The function opens the wanted RS232 port
*/
CAENRFIDErrorCodes
connectRS232 (char* filename, speed_t speed, char parity, tcflag_t bytesize, CAENRFIDHandle *handle);


/*
	getnbTags
	---------------------------------------------------------------------------
	Returns:
		the number of Tags ID sent by the reader
*/	
int getnbTags();
	
	
/*
        CAENRFID_Init
        -----------------------------------------------------------------------------
        Parameters:
            [in]  CommProtocol  : Communication port (see CAENRFIDPort enum).
            [in]  port 			: The COM port on which the reader is connected
									(eg. 1 for COM1)
			[in]  speed			: The speed of the connection
									(defined in termios.h, B115200 for example)
			[in]  parity		: The parity of the connection : 
									E for even, O for odd, N for no
			[in]  bytesize		: The number of bytes sent each time
									(defined in termios.h, CS8 for example)
            [out] Handle    	: The handle that identifies the device.
			[out] Protocol		: The answered protocol (see CAENRFIDProtocol enum)
        -----------------------------------------------------------------------------
        Returns:
            An error code about the execution of the function.
        -----------------------------------------------------------------------------
        Description:
            The function generates an opaque handle to identify a module
            attached to the PC.
*/
CAENRFIDErrorCodes
CAENRFID_Init (CAENRFIDPort CommProtocol, char* _port, speed_t speed, char parity, tcflag_t bytesize, CAENRFIDHandle *Handle, CAENRFIDProtocol *Protocol);

/*
    CAENRFID_End
    -----------------------------------------------------------------------------
    Parameters:
        [in]  Handle    : The handle that identifies the device.
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        Notifies the library the end of work and free the allocated
        resources.
*/
CAENRFIDErrorCodes 
CAENRFID_End(CAENRFIDHandle Handle);

/*
    CAENRFID_GetSWRelease
    -----------------------------------------------------------------------------
    Parameters:
        [out] SwRel : Returns the software release of the library.
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        Permits to read the software release of the library.
*/
CAENRFIDErrorCodes 
CAENRFID_GetSWRelease(char *SwRel);

/*
    CAENRFID_GetFWRelease.
    -----------------------------------------------------------------------------
    Parameters:
        [in]  Handle    : The handle that identifies the device.
        [out] FWRel : Returns the firmware release of the device.
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        Permits to read the firmware release loaded into the device.
*/
CAENRFIDErrorCodes
CAENRFID_GetFWRelease(CAENRFIDHandle Handle, char *FWRel);


/*
    CAENRFID_Inventory.
    -----------------------------------------------------------------------------
    Parameters:
        [in]  Handle    : The handle that identifies the device.
        [in]  LogicalSourceName : The name that identify the Logical Source
        [out] Tags  : Returns an array containing the tags read.
        [out] TagsNo    : Returns the number of tags in the array.
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        The function returns all the IDs of the tags under the reader field
        using all the available antennae. The Tags array contains The IDs
        together with other information related to the single ID such as the
        antenna under which is the ID and the format of the ID itself (see
        CAENRFIDTag struct for the details).
*/
CAENRFIDErrorCodes 
CAENRFID_Inventory(CAENRFIDHandle Handle, char *LogicalSourceName, CAENRFIDTag **Tags, int *TagsNo);

/*
    CAENRFID_SetPower.
    -----------------------------------------------------------------------------
    Parameters:
        [in]  Handle    : The handle that identifies the device.
                [in]  Power     : RF field power expressed in mW.
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        The function permits to set the RF field power relative to the antenna
        socket.
*/
CAENRFIDErrorCodes 
CAENRFID_SetPower(CAENRFIDHandle Handle, unsigned int Power);

/*
    CAENRFID_Read
    -----------------------------------------------------------------------------
    Parameters:
        [in]  Handle    : The handle that identifies the device.
        [in]  ID        : The tag ID.
        [in]  Address   : The address of the memory to read.
        [in]  Length    : The number of bytes to read.
        [out] Data      : The data read from the tag's memory.
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        This function allows to read Length bytes from the memory of a
        specific tag identified by the ID (regardless of its status) at the
        address specified by Address.
*/
CAENRFIDErrorCodes 
CAENRFID_Read(CAENRFIDHandle Handle, CAENRFIDTag *ID, int Address, int Length, void *Data,int type);

/*
    CAENRFID_Write
    -----------------------------------------------------------------------------
    Parameters:
        [in]  Handle    : The handle that identifies the device.
        [in]  ID        : The tag ID.
        [in]  Address   : The address of the memory to write.
        [in]  Length    : The number of bytes to write.
        [in]  Data      : The data to write in the tag's memory.
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        This function allows to write Length bytes to the memory of a
        specific tag identified by the ID (regardless of its status) at the
        address specified by Address.
*/
CAENRFIDErrorCodes 
CAENRFID_Write(CAENRFIDHandle Handle, CAENRFIDTag *ID, int Address, int Length, void *Data);

/*
    CAENRFID_Lock
    -----------------------------------------------------------------------------
    Parameters:
        [in]  Handle    : The handle that identifies the device.
        [in]  ID        : The tag ID.
        [in]  Address   : The address of the tag's memory.
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        This function allows to lockthe memory of a
        specific tag identified by the ID at the address specified by Address.
*/
 CAENRFIDErrorCodes 
CAENRFID_Lock(CAENRFIDHandle Handle, CAENRFIDTag *ID, short Address);

/*
    CAENRFID_AddReadPoint.
    -----------------------------------------------------------------------------
    Parameters:
        [in]  Handle         : The handle that identifies the device.
        [in]  SourceName     : The name of the Logical Source.
        [in]  ReadPoint      : The name of the Read Point.
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        The function permits to add a read point (antenna) to a logical source..
*/
CAENRFIDErrorCodes
CAENRFID_AddReadPoint(CAENRFIDHandle handle, char *SourceName, char *ReadPoint);

/*
    CAENRFID_RemoveReadPoint.
    -----------------------------------------------------------------------------
    Parameters:
        [in]  Handle         : The handle that identifies the device.
        [in]  SourceName     : The name of the Logical Source.
        [in]  ReadPoint      : The name of the Read Point.
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        The function permits to add a read point (antenna) to a logical source.
*/
CAENRFIDErrorCodes
CAENRFID_RemoveReadPoint(CAENRFIDHandle handle, char *SourceName, char *ReadPoint);


/*
    CAENRFID_GetPower.
    -----------------------------------------------------------------------------
    Parameters:
        [in]  Handle         : The handle that identifies the device.
        [out] Power          : The ERP power of the reader.
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        The function returns the value of the ERP power setting in the reader.
*/
CAENRFIDErrorCodes
CAENRFID_GetPower(CAENRFIDHandle Handle, unsigned int *Power);

/*
    CAENRFID_SetProtocol.
    -----------------------------------------------------------------------------
    Parameters:
        [in]  Handle         : The handle that identifies the device.
        [in]  Protocol       : The tag protocol to be set in the reader.
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        The function permits to change the tag protocol used by the reader.
*/
CAENRFIDErrorCodes
CAENRFID_SetProtocol (CAENRFIDHandle Handle, CAENRFIDProtocol Protocol);

/*
    CAENRFID_GetProtocol.
    -----------------------------------------------------------------------------
    Parameters:
        [in]  Handle         : The handle that identifies the device.
        [out] Protocol       : The tag protocol to be set in the reader.
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        The function permits to know what tag protocol is used by the reader.
*/
CAENRFIDErrorCodes
CAENRFID_GetProtocol(CAENRFIDHandle Handle, int *Protocol);

/*
    CAENRFID_GetReadPointStatus.
    -----------------------------------------------------------------------------
    Parameters:
        [in]  Handle         : The handle that identifies the device.
        [in]  ReadPoint      : The name of the Read Point.
        [out] Status         : The status of the read point.
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        The function permits to check the status of a read point.
*/
CAENRFIDErrorCodes
CAENRFID_GetReadPointStatus(CAENRFIDHandle Handle, char *ReadPoint, CAENRFIDReadPointStatus *Status);

/*
    CAENRFID_GetReadPointInSource.
    -----------------------------------------------------------------------------
    Parameters:
        [in]  Handle         : The handle that identifies the device.
        [in]  SourceName     : The name of the Logical Source.
        [in]  ReadPoint      : The name of the Read Point.
        [out] isPresent      : A flag indicating if the read point is associated
                               to the specified source.
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        The function permits to check if a read point is associated to a
        specified logical source that is, the read point is used within a read
        cycle performed in the source.
*/
CAENRFIDErrorCodes
CAENRFID_GetReadPointInSource(CAENRFIDHandle Handle, char *ReadPoint, char *SourceName, short *isPresent);

/*
    CAENRFID_SetModulation.
    -----------------------------------------------------------------------------
    Parameters:
        [in]  Handle    : The handle that identifies the device.
        [in]  TxRxCfg   : Modulation setting.
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        The function permits to control to choose the modulation (the bit
        rate of the transmission and receive).
*/


 CAENRFIDErrorCodes
CAENRFID_SetModulation(CAENRFIDHandle Handle, unsigned short TxRxCfg);

/*
    CAENRFID_GetModulation.
    -----------------------------------------------------------------------------
    Parameters:
        [in]  Handle    : The handle that identifies the device.
        [out]  TxRxCfg  : Modulation setting.
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        The function permits to retrieve the modulation (the bit
        rate of the transmission and receive).
*/
CAENRFIDErrorCodes 
CAENRFID_GetModulation(CAENRFIDHandle handle, unsigned short *TxRx);


/*
    CAENRFID_BlockWrite
    -----------------------------------------------------------------------------
    Parameters:
        [in]  Handle    : The handle that identifies the device.
        [in]  ID        : The tag ID.
        [in]  Address   : The address of the memory to write.
        [in]  Length    : The number of bytes to write.
        [in]  Data      : The data to write in the tag's memory.
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        This function allows to write Length bytes to the memory of a
        specific tag identified by the ID (regardless of its status) at the
        address specified by Address.
		This function doesn't work with semi-passive tags
*/

CAENRFIDErrorCodes
CAENRFID_BlockWrite(CAENRFIDHandle handle, CAENRFIDTag *ID, int Address, int Length, void *Data);


/*
    CAENRFID_SetRS232.
    -----------------------------------------------------------------------------
    Parameters:
        [in]  Handle		    : The handle that identifies the device.
        [in]  baud			    : The baudrate value.
        [in]  datab		        : The databit value.
        [in]  stopb             : The stopbit value.
        [in]  parity		    : The parity value.
        [in]  flowc		        : The flowcontrol value.
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        The function permits to configure the serial communication of the reader
*/
CAENRFIDErrorCodes
CAENRFID_SetRS232(CAENRFIDHandle handle, unsigned long baud, unsigned long datab, unsigned long stopb, CAENRFID_RS232_Parity parity, CAENRFID_RS232_FlowControl flowc);


/*
    CAENRFID_SetDateTime.
    -----------------------------------------------------------------------------
    Parameters:
        [in]  Handle         : The handle that identifies the device.
        [in]  datetime       : The current date ed time.
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        The function permits to set the date e the time in the reader.
*/
CAENRFIDErrorCodes
CAENRFID_SetDateTime(CAENRFIDHandle handle, char *datetime);

/*
    CAENRFID_GetIO.
    -----------------------------------------------------------------------------
    Parameters:
        [in]  Handle         : The handle that identifies the device.
        [out] IORegister     : The current IO Register.
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        The function permits to read the IO register.
*/
CAENRFIDErrorCodes
CAENRFID_GetIO(CAENRFIDHandle handle, unsigned int *IORegister);

/*
    CAENRFID_SetIO.
    -----------------------------------------------------------------------------
    Parameters:
        [in]  Handle         : The handle that identifies the device.
        [in]  IORegister     : The IO Register value.
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        The function permits to write the IO register.
*/
CAENRFIDErrorCodes
CAENRFID_SetIO(CAENRFIDHandle handle, unsigned int IORegister);


/*
    CAENRFID_GetIODirection.
    -----------------------------------------------------------------------------
    Parameters:
        [in]  Handle         : The handle that identifies the device.
        [out] IODirection     : The current IO Direction.
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        The function permits to read the IO direction of IORegister.
*/
CAENRFIDErrorCodes
CAENRFID_GetIODirection(CAENRFIDHandle handle, unsigned int *IODirection);

/*
    CAENRFID_SetIODirection.
    -----------------------------------------------------------------------------
    Parameters:
        [in]  Handle         : The handle that identifies the device.
        [in]  IODirection     : The IO Direction value.
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        The function permits to set the IO Direction for the IORegister.
*/
CAENRFIDErrorCodes
CAENRFID_SetIODirection(CAENRFIDHandle handle, unsigned int IODirection);

/*
    CAENRFID_SetSourceConfiguration.
    -----------------------------------------------------------------------------
    Parameters:
        [in]  Handle         : The handle that identifies the device.
        [in]  SourceName     : The Name of the Logical Source.
        [in]  parameter      : The parameter of Logical Source to configure.
        [in]  value          : The the value of the parameter.
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        The function permits to configure the Logical Source.
*/
CAENRFIDErrorCodes
CAENRFID_SetSourceConfiguration(CAENRFIDHandle handle, char *SourceName, CAENRFID_SOURCE_Parameter parameter, int value);

/*
    CAENRFID_GetSourceConfiguration.
    -----------------------------------------------------------------------------
    Parameters:
        [in]  Handle         : The handle that identifies the device.
        [in]  SourceName     : The Name of the Logical Source.
        [in]  parameter      : The parameter of Logical Source to configure.
        [out] value          : The the value of the parameter.
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        The function permits to get the value of the Logical Source configuration.
*/
CAENRFIDErrorCodes
CAENRFID_GetSourceConfiguration(CAENRFIDHandle handle, char *SourceName, CAENRFID_SOURCE_Parameter parameter, int *pvalue);


/*
    CAENRFID_GetReaderInfo
    -----------------------------------------------------------------------------
    Parameters:
        [out] Model : 		Returns the model of the reader.
        [out] SerialNum : 	Returns the Serial number of the reader.
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        Permits to read the Model and the Serial number of the Reader.
*/

CAENRFIDErrorCodes
CAENRFID_GetReaderInfo(CAENRFIDHandle handle,char *Model, char *SerialNum);

/*
    CAENRFID_InventoryTag.
    -----------------------------------------------------------------------------
    Parameters:
        [in]  Handle    : The handle that identifies the device.
        [in]  LogicalSourceName : The name that identify the Logical Source
        [out] Receive  : Returns an array containing the tags read.
        [out] Size     : Returns the number of tags in the array.
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        The function returns all the IDs of the tags under the reader field
        using all the available antennae. The Tags array contains The IDs
        together with other information related to the single ID such as the
        antenna under which is the ID and the format of the ID itself (see
        CAENRFIDTag struct for the details).
*/

CAENRFIDErrorCodes
CAENRFID_InventoryTag (CAENRFIDHandle handle, char *SourceName, CAENRFIDTag **Receive, int *Size);

/*
    CAENRFID_GetLBTMode.
    -----------------------------------------------------------------------------
    Parameters:
        [in]  Handle         : The handle that identifies the device.
        [out] LBTMode		 : The ListenBeforeTalk Mode of the reader.
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        The function permits to get the The ListenBeforeTalk Mode of the reader.
*/
CAENRFIDErrorCodes
CAENRFID_GetLBTMode(CAENRFIDHandle handle, unsigned short *LBTMode);

/*
    CAENRFID_SetLBTMode.
    -----------------------------------------------------------------------------
    Parameters:
        [in]  Handle         : The handle that identifies the device.
        [in]  LBTMode        : The ListenBeforeTalk Mode of the reader.
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        The function permits to change the The ListenBeforeTalk Mode of the reader.
*/

CAENRFIDErrorCodes
CAENRFID_SetLBTMode(CAENRFIDHandle handle, unsigned short LBTMode);

/*
    CAENRFID_FreeTagsMemory.
    -----------------------------------------------------------------------------
    Parameters:
        [in]  Tags         : Reference to CAENRFIDTag obtained from CAENRFIDInventory 
							 or CAENRFID_InventoryTag.
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        The function permits to free the allocated by CAENRFIDInventory or 
		by CAENRFID_InventoryTag.
*/
void
CAENRFID_FreeTagsMemory(CAENRFIDTag **Tags);

/*
    CAENRFID_GetRFChannel.
    -----------------------------------------------------------------------------
    Parameters:
		[in]  handle    : The handle that identifies the device.
        [out] RFChannel : The RFChannel
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        The function permits to retrieve the RFChanel.
*/

CAENRFIDErrorCodes
CAENRFID_GetRFChannel(CAENRFIDHandle handle, unsigned short *RFChannel);

/*
    CAENRFID_SetRFChannel.
    -----------------------------------------------------------------------------
    Parameters:
		[in]  handle    : The handle that identifies the device.
        [in]  RFChannel : The RFChannel
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        The function permits to set the RFChanel.
*/
CAENRFIDErrorCodes
CAENRFID_SetRFChannel(CAENRFIDHandle handle, unsigned short RFChannel);

/*
    CAENRFID_GetRFRegulation.
    -----------------------------------------------------------------------------
    Parameters:
		[in]  handle		: The handle that identifies the device.
        [out] RFRegulation  : The RFRegulation
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        The function permits to retrieve the RFRegulation.
*/
CAENRFIDErrorCodes
CAENRFID_GetRFRegulation(CAENRFIDHandle handle, unsigned short *RFRegulation);

/*
    CAENRFID_SetRFRegulation.
    -----------------------------------------------------------------------------
    Parameters:
		[in]  handle		: The handle that identifies the device.
        [in]  RFRegulation  : The RFRegulation
    -----------------------------------------------------------------------------
    Returns:
        An error code about the execution of the function.
    -----------------------------------------------------------------------------
    Description:
        The function permits to set the RFRegulation.
*/
CAENRFIDErrorCodes
CAENRFID_SetRFRegulation(CAENRFIDHandle handle, unsigned short RFRegulation);


#ifdef __cplusplus
}
#endif /* __cplusplus*/

#endif /* __CAENRFIDLIB_H*/
